@@ -496,6 +496,10 @@ static void init_window(ANativeWindow *window) {
   _display.bytes_per_pixel = 2;
   _display.rgbinfo = rgbinfo;
 
+  /*
+   * If AndroidBitmapFormat is changed (such as WINDOW_FORMAT_RGBA_8888),
+   * 'rgbinfo' structure and ALPHA_TO_PIXEL() should be changed, too.
+   */
   ANativeWindow_setBuffersGeometry(window, 0, 0, WINDOW_FORMAT_RGB_565);
 
   if (_display.buf.bits) {
@@ -514,6 +518,10 @@ static void init_window(ANativeWindow *window) {
   }
 }
 
+/* ui_event_source.c */
+void ui_activity_pause(void);
+void ui_activity_resume(void);
+
 static void on_app_cmd(struct android_app *app, int32_t cmd) {
   switch (cmd) {
     case APP_CMD_SAVE_STATE:
@@ -586,6 +594,22 @@ static void on_app_cmd(struct android_app *app, int32_t cmd) {
         ASensorEventQueue_disableSensor(_display.sensor_evqueue, _display.accel_sensor);
       }
 
+      break;
+
+    case APP_CMD_PAUSE:
+#ifdef DEBUG
+      bl_debug_printf("PAUSE\n");
+#endif
+      ui_activity_pause();
+
+      break;
+
+    case APP_CMD_RESUME:
+#ifdef DEBUG
+      bl_debug_printf("RESUME\n");
+#endif
+      ui_activity_resume();
+
       break;
   }
 }
@@ -622,10 +646,13 @@ void ui_display_close_all(void) {
     }
 
     free(_disp.roots);
+    _disp.roots = NULL;
+    _disp.num_roots = 0;
 
-    memset(&_display, 0, sizeof(_display));
     /* DISP_IS_INITED is false from here. */
-    memset(&_disp, 0, sizeof(_disp));
+    _disp.display = NULL;
+
+    /* Other members in _disp is reset in ui_display_init() */
   }
 }
 
@@ -1010,6 +1037,7 @@ void ui_display_send_text_selection(u_char *sel_data, size_t sel_len) {
 
 void ui_display_show_dialog(char *server, char *privkey) {
   char *server_dup;
+  char *proto;
   char *user;
   char *host;
   char *port;
@@ -1023,8 +1051,8 @@ void ui_display_show_dialog(char *server, char *privkey) {
 
   if (server == NULL ||
       !(server_dup = alloca(strlen(server) + 1)) ||
-      !bl_parse_uri(NULL, &user, &host, &port, NULL, &encoding, strcpy(server_dup, server))) {
-    user = host = port = encoding = NULL;
+      !bl_parse_uri(&proto, &user, &host, &port, NULL, &encoding, strcpy(server_dup, server))) {
+    proto = user = host = port = encoding = NULL;
   }
 
   if (!privkey) {
@@ -1041,7 +1069,8 @@ void ui_display_show_dialog(char *server, char *privkey) {
                                              "showConnectDialog",
                                              "(Ljava/lang/String;Ljava/lang/String;"
                                              "Ljava/lang/String;Ljava/lang/String;"
-                                             "Ljava/lang/String;)V"),
+                                             "Ljava/lang/String;Ljava/lang/String;)V"),
+                         proto ? (*env)->NewStringUTF(env, proto) : NULL,
                          user ? (*env)->NewStringUTF(env, user) : NULL,
                          host ? (*env)->NewStringUTF(env, host) : NULL,
                          port ? (*env)->NewStringUTF(env, port) : NULL,
