@@ -44,7 +44,6 @@
 #include <vt_iscii.h>
 #include <ui_im.h>
 
-#include "../im_common.h"
 #include "../im_info.h"
 
 #if 0
@@ -84,7 +83,6 @@ typedef struct im_kbd {
   vt_isciikey_state_t isciikey_state;
 
   ef_parser_t *parser;
-  ef_conv_t *conv;
 
 } im_kbd_t;
 
@@ -312,10 +310,6 @@ static void destroy(ui_im_t *im) {
     (*kbd->parser->destroy)(kbd->parser);
   }
 
-  if (kbd->conv) {
-    (*kbd->conv->destroy)(kbd->conv);
-  }
-
   ref_count--;
 
 #ifdef IM_KBD_DEBUG
@@ -336,7 +330,6 @@ static int key_event_arabic_hebrew(ui_im_t *im, u_char key_char, KeySym ksym, XK
   im_kbd_t *kbd;
   size_t len;
   u_char *c;
-  u_char conv_buf[10];
 
   kbd = (im_kbd_t*)im;
 
@@ -369,21 +362,7 @@ static int key_event_arabic_hebrew(ui_im_t *im, u_char key_char, KeySym ksym, XK
     len = strlen(c);
   }
 
-  (*kbd->parser->init)(kbd->parser);
-  (*kbd->parser->set_str)(kbd->parser, c, len);
-
-  (*kbd->conv->init)(kbd->conv);
-
-  while (!kbd->parser->is_eos) {
-    len = (*kbd->conv->convert)(kbd->conv, conv_buf, sizeof(conv_buf), kbd->parser);
-
-    if (len == 0) {
-      /* finished converting */
-      break;
-    }
-
-    (*kbd->im.listener->write_to_term)(kbd->im.listener->self, conv_buf, len);
-  }
+  (*kbd->im.listener->write_to_term)(kbd->im.listener->self, c, len, kbd->parser);
 
   return 0;
 }
@@ -392,7 +371,6 @@ static int key_event_iscii(ui_im_t *im, u_char key_char, KeySym ksym, XKeyEvent
   im_kbd_t *kbd;
   u_char buf[512];
   size_t len;
-  u_char conv_buf[10];
 
   kbd = (im_kbd_t*)im;
 
@@ -410,21 +388,7 @@ static int key_event_iscii(ui_im_t *im, u_char key_char, KeySym ksym, XKeyEvent
 
   len = (*syms->vt_convert_ascii_to_iscii)(kbd->isciikey_state, buf, sizeof(buf), &key_char, 1);
 
-  (*kbd->parser->init)(kbd->parser);
-  (*kbd->parser->set_str)(kbd->parser, buf, len);
-
-  (*kbd->conv->init)(kbd->conv);
-
-  while (!kbd->parser->is_eos) {
-    len = (*kbd->conv->convert)(kbd->conv, conv_buf, sizeof(conv_buf), kbd->parser);
-
-    if (len == 0) {
-      /* finished converting */
-      break;
-    }
-
-    (*kbd->im.listener->write_to_term)(kbd->im.listener->self, conv_buf, len);
-  }
+  (*kbd->im.listener->write_to_term)(kbd->im.listener->self, buf, len, kbd->parser);
 
   return 0;
 }
@@ -460,7 +424,6 @@ static int switch_mode(ui_im_t *im) {
 #ifdef IM_KBD_DEBUG
       bl_debug_printf(BL_DEBUG_TAG " switched to inscript.\n");
 #endif
-
     } else if (kbd->mode == KBD_MODE_ISCII_INSCRIPT) {
       kbd->isciikey_state = (*syms->vt_isciikey_state_new)(0);
       kbd->mode = KBD_MODE_ISCII_PHONETIC;
@@ -476,7 +439,7 @@ static int switch_mode(ui_im_t *im) {
 #endif
     }
 
-    if ((kbd->type == KBD_MODE_ISCII_INSCRIPT || kbd->type == KBD_MODE_ISCII_PHONETIC) &&
+    if ((kbd->mode == KBD_MODE_ISCII_INSCRIPT || kbd->mode == KBD_MODE_ISCII_PHONETIC) &&
         (kbd->isciikey_state == NULL)) {
 #ifdef DEBUG
       bl_warn_printf(BL_DEBUG_TAG " vt_isciikey_state_new() failed.\n");
@@ -604,7 +567,6 @@ ui_im_t *im_kbd_new(u_int64_t magic, vt_char_encoding_t term_encoding,
   kbd->mode = KBD_MODE_ASCII;
   kbd->isciikey_state = NULL;
   kbd->parser = NULL;
-  kbd->conv = NULL;
 
   if (kbd->type == KBD_TYPE_ARABIC || kbd->type == KBD_TYPE_HEBREW) {
     if (!(kbd->parser = ef_utf16_parser_new())) {
@@ -626,10 +588,6 @@ ui_im_t *im_kbd_new(u_int64_t magic, vt_char_encoding_t term_encoding,
     }
   }
 
-  if (!(kbd->conv = (*syms->vt_char_encoding_conv_new)(term_encoding))) {
-    goto error;
-  }
-
   /*
    * set methods of ui_im_t
    */
