@@ -7,6 +7,7 @@
 #include <pobl/bl_str.h> /* strdup */
 #include <pobl/bl_mem.h> /* free */
 #include <pobl/bl_debug.h>
+#include <pobl/bl_util.h> /* BL_ARRAY_SIZE */
 #include <glib.h>
 #include <c_intl.h>
 
@@ -14,6 +15,7 @@
 #include "gtkxlfdsel.c"
 #endif
 
+#include "mc_compat.h"
 #include "mc_combo.h"
 #include "mc_flags.h"
 #include "mc_radio.h"
@@ -164,7 +166,7 @@ static char *new_fontsize = NULL;
 static char *old_fontsize = NULL;
 static int is_fontsize_changed;
 
-static char *new_fontname_list[sizeof(cs_info_table) / sizeof(cs_info_table[0])];
+static char *new_fontname_list[BL_ARRAY_SIZE(cs_info_table)];
 static int dont_change_new_fontname_list = 0;
 static int selected_cs = 0; /* 0 = DEFAULT */
 static GtkWidget *fontcs_entry;
@@ -183,7 +185,7 @@ static char *noconv_areas;
 static void reset_fontname_list(void) {
   int count;
 
-  for (count = 0; count < sizeof(cs_info_table) / sizeof(cs_info_table[0]); count++) {
+  for (count = 0; count < BL_ARRAY_SIZE(cs_info_table); count++) {
     free(new_fontname_list[count]);
     new_fontname_list[count] = NULL;
   }
@@ -214,7 +216,7 @@ static const char *get_correct_cs(int idx) {
     return NULL;
   } else if (1 <= idx && idx <= 15) {
     return unicode_names[idx - 1];
-  } else if (idx < sizeof(cs_info_table) / sizeof(cs_info_table[0])) {
+  } else if (idx < BL_ARRAY_SIZE(cs_info_table)) {
     return cs_info_table[idx].cs;
   } else {
     return NULL;
@@ -243,7 +245,7 @@ static char *get_font_file(void) {
   }
 }
 
-static gint aa_flag_checked(GtkWidget *widget, gpointer data) {
+static void aa_flag_checked(GtkWidget *widget, gpointer data) {
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
     if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(xft_flag)) &&
         !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cairo_flag))) {
@@ -256,11 +258,9 @@ static gint aa_flag_checked(GtkWidget *widget, gpointer data) {
                            NULL, NULL));
     }
   }
-
-  return 1;
 }
 
-static gint xft_flag_checked(GtkWidget *widget, gpointer data) {
+static void xft_flag_checked(GtkWidget *widget, gpointer data) {
   if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) &&
       !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cairo_flag))) {
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(aa_flag), 0);
@@ -275,11 +275,9 @@ static gint xft_flag_checked(GtkWidget *widget, gpointer data) {
       GTK_ENTRY(fontname_entry),
       g_locale_to_utf8(mc_get_font_name(get_font_file(), get_correct_cs(selected_cs)), -1, NULL,
                        NULL, NULL));
-
-  return 1;
 }
 
-static gint cairo_flag_checked(GtkWidget *widget, gpointer data) {
+static void cairo_flag_checked(GtkWidget *widget, gpointer data) {
   if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)) &&
       !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(xft_flag))) {
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(aa_flag), 0);
@@ -294,18 +292,14 @@ static gint cairo_flag_checked(GtkWidget *widget, gpointer data) {
       GTK_ENTRY(fontname_entry),
       g_locale_to_utf8(mc_get_font_name(get_font_file(), get_correct_cs(selected_cs)), -1, NULL,
                        NULL, NULL));
-
-  return 1;
 }
 
-static gint vcol_flag_checked(GtkWidget *widget, gpointer data) {
+static void vcol_flag_checked(GtkWidget *widget, gpointer data) {
   reset_fontname_list();
   gtk_entry_set_text(
       GTK_ENTRY(fontname_entry),
       g_locale_to_utf8(mc_get_font_name(get_font_file(), get_correct_cs(selected_cs)), -1, NULL,
                        NULL, NULL));
-
-  return 1;
 }
 
 static void vertical_mode_changed(void) {
@@ -323,15 +317,13 @@ static void vertical_mode_changed(void) {
   }
 }
 
-static gint fontsize_selected(GtkWidget *widget, gpointer data) {
+static void fontsize_selected(GtkWidget *widget, gpointer data) {
   g_free(new_fontsize);
   new_fontsize = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
 
 #ifdef __DEBUG
   bl_debug_printf(BL_DEBUG_TAG " %s font size is selected.\n", new_fontsize);
 #endif
-
-  return 1;
 }
 
 static void specify_width(GtkWidget *widget, int flag) {
@@ -441,7 +433,7 @@ static void fontcs_map(GtkWidget *widget, gpointer data) {
 
     selected_cs = 1; /* UNICODE */
   } else {
-    for (count = 0; count < sizeof(cs_info_table) / sizeof(cs_info_table[0]); count++) {
+    for (count = 0; count < BL_ARRAY_SIZE(cs_info_table); count++) {
       if (strcmp(cs_info_table[count].cs, "JISX0201_KATA") == 0) {
         break;
       } else if (compare(encoding, cs_info_table[count].cs)) {
@@ -475,13 +467,13 @@ static void font_policy_changed(void) {
   }
 }
 
-static gint fontcs_selected(GtkWidget *widget, gpointer data) {
+static void fontcs_selected(GtkWidget *widget, gpointer data) {
   const char *cs;
   int count;
 
   cs = gtk_entry_get_text(GTK_ENTRY(widget));
 
-  for (count = 0; count < sizeof(cs_info_table) / sizeof(cs_info_table[0]); count++) {
+  for (count = 0; count < BL_ARRAY_SIZE(cs_info_table); count++) {
     if (strcmp(cs, cs_info_table[count].cs) == 0) {
       if (selected_cs != count) {
 #if 0
@@ -496,13 +488,9 @@ static gint fontcs_selected(GtkWidget *widget, gpointer data) {
 #if 0
         bl_debug_printf("After: cs %s fontname %s\n", cs, new_fontname_list[selected_cs]);
 #endif
-
-        return 1;
       }
     }
   }
-
-  return 0;
 }
 
 #if defined(GDK_WINDOWING_X11) && !GTK_CHECK_VERSION(2, 90, 0)
@@ -611,13 +599,22 @@ static void select_xlfd_font(GtkWidget *widget, gpointer label) {
 
 #endif
 
-static char *my_gtk_font_selection_dialog_get_font_name(GtkFontSelectionDialog *dialog) {
+#if GTK_CHECK_VERSION(4, 0, 0)
+static char *my_gtk_font_chooser_get_font_name(GtkFontChooser *dialog)
+#else
+static char *my_gtk_font_selection_dialog_get_font_name(GtkFontSelectionDialog *dialog)
+#endif
+{
   char *str;
   int count;
   char *p;
   char *dup_str;
 
+#if GTK_CHECK_VERSION(4, 0, 0)
+  str = gtk_font_chooser_get_font(dialog);
+#else
   str = gtk_font_selection_dialog_get_font_name(dialog);
+#endif
 
   if ((dup_str = malloc(strlen(str) * 2 + 1)) == NULL) {
     free(str);
@@ -690,11 +687,43 @@ static void fontname_entry_edit(GtkWidget *widget, gpointer p) {
   }
 }
 
+#if GTK_CHECK_VERSION(4, 0, 0)
+static void dialog_cb(GtkDialog *dialog, int response_id, gpointer user_data) {
+  if (response_id == GTK_RESPONSE_OK) {
+    char *name = my_gtk_font_chooser_get_font_name(GTK_FONT_CHOOSER(dialog));
+
+    if (name != NULL) {
+      gtk_entry_set_text(GTK_ENTRY(fontname_entry), name);
+      free(name);
+    }
+  }
+  gtk_window_destroy(GTK_WINDOW(dialog));
+}
+#endif
+
 static void select_fc_font(GtkWidget *widget, gpointer p) {
   GtkWidget *dialog;
   char *font_name;
-  GtkResponseType result;
 
+#if GTK_CHECK_VERSION(4, 0, 0)
+  dialog = gtk_font_chooser_dialog_new("Select Font", NULL);
+
+  font_name = get_gtk_font_name(gtk_entry_get_text(GTK_ENTRY(fontname_entry)));
+  if (!font_name || !*font_name) {
+    if ((font_name = malloc(6 + strlen(new_fontsize))) == NULL) {
+      return;
+    }
+
+    sprintf(font_name, "Sans %s", new_fontsize);
+  }
+  gtk_font_chooser_set_font(GTK_FONT_CHOOSER(dialog), font_name);
+
+  free(font_name);
+
+  gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
+  g_signal_connect(dialog, "response", G_CALLBACK(dialog_cb), NULL);
+  gtk_widget_show(dialog);
+#else
   dialog = gtk_font_selection_dialog_new("Select Font");
 
   font_name = get_gtk_font_name(gtk_entry_get_text(GTK_ENTRY(fontname_entry)));
@@ -713,13 +742,17 @@ static void select_fc_font(GtkWidget *widget, gpointer p) {
 
   free(font_name);
 
-  result = gtk_dialog_run(GTK_DIALOG(dialog));
-  if (result == GTK_RESPONSE_OK) {
-    gtk_entry_set_text(GTK_ENTRY(fontname_entry), my_gtk_font_selection_dialog_get_font_name(
-                                                      GTK_FONT_SELECTION_DIALOG(dialog)));
+  if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
+    char *name = my_gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(dialog));
+
+    if (name != NULL) {
+      gtk_entry_set_text(GTK_ENTRY(fontname_entry), name);
+      free(name);
+    }
   }
 
   gtk_widget_destroy(dialog);
+#endif
 }
 
 static void select_font(GtkWidget *widget, gpointer p) {
@@ -782,7 +815,7 @@ GtkWidget *mc_font_config_widget_new(void) {
 
   combo =
       mc_combo_new_with_width(_("Font size (pixels)"), fontlist,
-                              sizeof(fontlist) / sizeof(fontlist[0]), new_fontsize, 1, 30, &entry);
+                              BL_ARRAY_SIZE(fontlist), new_fontsize, 1, 30, &entry);
   g_signal_connect(entry, "changed", G_CALLBACK(fontsize_selected), NULL);
   gtk_box_pack_start(GTK_BOX(hbox), combo, FALSE, FALSE, 0);
 #if GTK_CHECK_VERSION(2, 12, 0)
@@ -836,10 +869,10 @@ GtkWidget *mc_font_config_widget_new(void) {
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_widget_show(hbox);
 
-  if ((cslist = alloca(sizeof(char*) * sizeof(cs_info_table) / sizeof(cs_info_table[0])))) {
+  if ((cslist = alloca(sizeof(char*) * sizeof(cs_info_table)))) {
     int count;
 
-    for (count = 0; count < sizeof(cs_info_table) / sizeof(cs_info_table[0]); count++) {
+    for (count = 0; count < BL_ARRAY_SIZE(cs_info_table); count++) {
       cslist[count] = cs_info_table[count].cs;
     }
 
@@ -949,7 +982,7 @@ void mc_update_font_misc(void) {
 void mc_update_font_name(mc_io_t io) {
   size_t count;
 
-  for (count = 0; count < sizeof(cs_info_table) / sizeof(cs_info_table[0]); count++) {
+  for (count = 0; count < BL_ARRAY_SIZE(cs_info_table); count++) {
     if (new_fontname_list[count]) {
       mc_set_font_name(io, get_font_file(), get_correct_cs(count), new_fontname_list[count]);
 
