@@ -505,10 +505,38 @@ static int open_display(u_int depth /* used on luna68k alone. */
     _display.rgbinfo.g_offset = vinfo2.fbi_subtype.fbi_rgbmasks.green_offset;
     _display.rgbinfo.b_offset = vinfo2.fbi_subtype.fbi_rgbmasks.blue_offset;
 
+    /*
+     * XXX
+     * wscons does not correctly set alpha size and offset.
+     * Therefore, we need to guess them below.
+     */
+    {
+      int r = _display.rgbinfo.r_offset,
+          g = _display.rgbinfo.g_offset,
+          b = _display.rgbinfo.b_offset;
+      if (_display.rgbinfo.r_limit != 0 ||
+          _display.rgbinfo.g_limit != 0 ||
+          _display.rgbinfo.b_limit != 0 ||
+          r % 8 != 0 || g % 8 != 0 || b % 8 != 0 ||
+          r == g || g == b || b == r ||
+          r > 24 || g > 24 || b > 24) {
+        _display.rgbinfo.a_limit = 8;
+        _display.rgbinfo.a_offset = 0;
+      } else {
+        int a, mask = (1 << (r / 8)) | (1 << (g / 8)) | (1 << (b / 8));
+        for (a = 0; a < 4; a++) {
+          if ((mask & (1 << a)) == 0)
+            break;
+        }
+        _display.rgbinfo.a_limit = 0;
+        _display.rgbinfo.a_offset = a * 8;
+      }
+    }
+
 #ifdef DEBUG
-    bl_debug_printf("FBINFO: (limit)r%d g%d b%d (offset)r%d g%d b%d\n", _display.rgbinfo.r_limit,
-                    _display.rgbinfo.g_limit, _display.rgbinfo.b_limit, _display.rgbinfo.r_offset,
-                    _display.rgbinfo.g_offset, _display.rgbinfo.b_offset);
+    bl_debug_printf("FBINFO: (limit)r%d g%d b%d a%d (offset)r%d g%d b%d a%d\n",
+      _display.rgbinfo.r_limit, _display.rgbinfo.g_limit, _display.rgbinfo.b_limit, _display.rgbinfo.a_limit,
+      _display.rgbinfo.r_offset, _display.rgbinfo.g_offset, _display.rgbinfo.b_offset, _display.rgbinfo.a_offset);
 #endif
   }
 #endif
@@ -670,7 +698,7 @@ error:
   return 0;
 }
 
-static int receive_mouse_event(void) {
+static int receive_mouse_event(int fd) {
   struct wscons_event ev;
   ssize_t len;
 
@@ -678,7 +706,7 @@ static int receive_mouse_event(void) {
     return 0;
   }
 
-  while ((len = read(_mouse.fd, memset(&ev, 0, sizeof(ev)), sizeof(ev))) > 0) {
+  while ((len = read(fd, memset(&ev, 0, sizeof(ev)), sizeof(ev))) > 0) {
 #ifdef DEBUG
     bl_debug_printf(BL_DEBUG_TAG " MOUSE event (len)%d (type)%d (val)%d\n", len, ev.type, ev.value);
 #endif
@@ -867,8 +895,8 @@ static int receive_mouse_event(void) {
   return 1;
 }
 
-static int receive_key_event(void) {
-  if (_display.fd == STDIN_FILENO) {
+static int receive_key_event(int fd) {
+  if (fd == STDIN_FILENO) {
     return receive_stdin_key_event();
   } else {
     ssize_t len;
@@ -878,7 +906,7 @@ static int receive_key_event(void) {
       return 0;
     }
 
-    while ((len = read(_display.fd, memset(&ev, 0, sizeof(ev)), sizeof(ev))) > 0) {
+    while ((len = read(fd, memset(&ev, 0, sizeof(ev)), sizeof(ev))) > 0) {
 #ifdef DEBUG
       bl_debug_printf(BL_DEBUG_TAG " KEY event (len)%d (type)%d (val)%d\n", len, ev.type, ev.value);
 #endif
