@@ -21,11 +21,7 @@
 
 /* --- static variables --- */
 
-#ifdef USE_WAYLAND
-static char *default_display = "wayland-0";
-#else
 static char *default_display = "";
-#endif
 
 /* --- static functions --- */
 
@@ -333,6 +329,8 @@ void ui_prepare_for_main_config(bl_conf_t *conf) {
                   "screen display at resize [wrap]");
   bl_conf_add_opt(conf, '\0', "recvdir", 0, "receive_directory",
                   "directory to save received files [~/.mlterm/recv]");
+  bl_conf_add_opt(conf, '\0', "fk", 1, "format_other_keys",
+                  "send modified keys as parameter for CSI u [false]");
 #ifdef USE_IM_CURSOR_COLOR
   bl_conf_add_opt(conf, '\0', "imcolor", 0, "im_cursor_color",
                   "cursor color when input method is activated. [false]");
@@ -360,6 +358,10 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
       !(main_config->disp_name = strdup(value)))
 #endif
   {
+    /*
+     * default_display ("") is replaced by getenv("WAYLAND_DISPLAY") on wayland.
+     * (See ui_display_open() in wayland/ui_display.c)
+     */
     main_config->disp_name = default_display;
 #ifdef USE_XLIB
     /* In case of starting mlterm with "-j genuine" option without X server. */
@@ -686,16 +688,22 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     }
   }
 
-#if defined(__APPLE__) || defined(USE_BEOS)
+#ifdef USE_BEOS
   main_config->use_login_shell = 1;
 #else
   main_config->use_login_shell = 0;
 #endif
 
   if ((value = bl_conf_get_value(conf, "use_login_shell"))) {
+#ifdef USE_BEOS
+    if (strcmp(value, "false") == 0) {
+      main_config->use_login_shell = 0;
+    }
+#else
     if (strcmp(value, "true") == 0) {
       main_config->use_login_shell = 1;
     }
+#endif
   }
 
   if ((value = bl_conf_get_value(conf, "big5_buggy"))) {
@@ -708,7 +716,7 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
 
   main_config->use_mdi = 1;
 
-#if !defined(USE_QUARTZ) && !defined(USE_BEOS)
+#if !defined(USE_QUARTZ) && !defined(USE_BEOS) /* see ui_layout.[ch] */
   if ((value = bl_conf_get_value(conf, "use_mdi"))) {
     if (strcmp(value, "false") == 0) {
       main_config->use_mdi = 0;
@@ -1408,6 +1416,14 @@ void ui_main_config_init(ui_main_config_t *main_config, bl_conf_t *conf, int arg
     vt_set_resize_mode(vt_get_resize_mode_by_name(value));
   }
 
+  if ((value = bl_conf_get_value(conf, "format_other_keys"))) {
+    int flag = true_or_false(value);
+
+    if (flag != -1) {
+      vt_set_format_other_keys(flag);
+    }
+  }
+
   if ((value = bl_conf_get_value(conf, "receive_directory"))) {
     vt_set_recv_dir(value);
   }
