@@ -332,6 +332,7 @@ static int open_pty_intern(vt_term_t *term, char *cmd_path, char **cmd_argv,
 #ifdef __APPLE__
     char *user;
 
+    /* Terminal.app uses /usr/bin/login by default. */
     if ((user = getenv("USER")) && (cmd_argv = alloca(sizeof(char *) * 4))) {
       cmd_argv[0] = cmd_path = "login";
       cmd_argv[1] = "-fp";
@@ -779,6 +780,7 @@ static void open_pty(void *p, ui_screen_t *screen, char *dev) {
     vt_char_encoding_t encoding;
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
     char *default_server;
+    char *privkey;
     char *new_cmd_line;
     char *new_cmd_line_dup;
     char *cmd_path;
@@ -799,6 +801,8 @@ static void open_pty(void *p, ui_screen_t *screen, char *dev) {
     if (!main_config.show_dialog) {
       default_server = main_config.default_server;
       main_config.default_server = vt_term_get_uri(screen->term);
+      privkey = main_config.private_key;
+      main_config.private_key = vt_term_get_privkey(screen->term);
     }
 
     if ((new_cmd_line = vt_term_get_cmd_line(screen->term)) &&
@@ -829,6 +833,7 @@ static void open_pty(void *p, ui_screen_t *screen, char *dev) {
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
     if (!main_config.show_dialog) {
       main_config.default_server = default_server;
+      main_config.private_key = privkey;
     }
 
     if (new_cmd_line) {
@@ -956,6 +961,7 @@ static void open_cloned_screen(ui_screen_t *cur_screen, ui_layout_t *layout, int
   vt_char_encoding_t encoding;
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
   char *default_server;
+  char *privkey;
   char *new_cmd_line;
   char *new_cmd_line_dup;
   char *cmd_path;
@@ -968,6 +974,8 @@ static void open_cloned_screen(ui_screen_t *cur_screen, ui_layout_t *layout, int
   if (!show_dialog) {
     default_server = main_config.default_server;
     main_config.default_server = vt_term_get_uri(cur_screen->term);
+    privkey = main_config.private_key;
+    main_config.private_key = vt_term_get_privkey(cur_screen->term);
   }
 
   if ((new_cmd_line = vt_term_get_cmd_line(cur_screen->term)) &&
@@ -998,6 +1006,7 @@ static void open_cloned_screen(ui_screen_t *cur_screen, ui_layout_t *layout, int
 #if defined(USE_WIN32API) || defined(USE_LIBSSH2)
   if (!show_dialog) {
     main_config.default_server = default_server;
+    main_config.private_key = privkey;
   }
 
   if (new_cmd_line) {
@@ -1298,7 +1307,7 @@ int ui_screen_manager_init(char *_mlterm_version, u_int _depth, u_int _max_scree
     char key3[] = "Button3";
     char *keys[] = {key0, key1, key2, key3};
 
-    for (count = 0; count < sizeof(keys) / sizeof(keys[0]); count++) {
+    for (count = 0; count < BL_ARRAY_SIZE(keys); count++) {
       if (main_config.shortcut_strs[count]) {
         ui_shortcut_parse(&shortcut, keys[count], main_config.shortcut_strs[count]);
       }
