@@ -47,6 +47,7 @@ static ui_wlserv_t **wlservs;
 static u_int num_displays;
 static ui_display_t **displays;
 static int rotate_display;
+static int use_clipboard = 1;
 int kbd_repeat_1 = DEFAULT_KEY_REPEAT_1;
 int kbd_repeat_N = DEFAULT_KEY_REPEAT_N;
 
@@ -755,8 +756,9 @@ static void pointer_motion(void *data, struct wl_pointer *pointer,
   ui_wlserv_t *wlserv = data;
   ui_display_t *disp;
 
-#ifdef ____DEBUG
-  bl_debug_printf("pointer_motion (time: %d)\n", time);
+#ifdef __DEBUG
+  bl_debug_printf("pointer_motion (time: %d) x %d y %d\n", time, wl_fixed_to_int(sx_w),
+                  wl_fixed_to_int(sy_w));
 #endif
   disp = surface_to_display(wlserv->current_pointer_surface);
 
@@ -2021,7 +2023,7 @@ static ui_wlserv_t *open_wl_display(char *name) {
     int count;
     int has_cursor = 0;
 
-    for (count = 0; count < sizeof(names) / sizeof(names[0]); count++) {
+    for (count = 0; count < BL_ARRAY_SIZE(names); count++) {
       if ((wlserv->cursor[count] = wl_cursor_theme_get_cursor(wlserv->cursor_theme,
                                                               names[count]))) {
         has_cursor = 1;
@@ -2488,6 +2490,12 @@ ui_display_t *ui_display_open(char *disp_name, u_int depth) {
   displays = p;
   displays[num_displays] = disp;
 
+  if (*disp_name == '\0') {
+    if ((disp_name = getenv("WAYLAND_DISPLAY")) == NULL) {
+      disp_name = "wayland-0";
+    }
+  }
+
   for (count = 0; count < num_displays; count++) {
     if (strcmp(displays[count]->name, disp_name) == 0) {
       disp->selection_owner = displays[count]->selection_owner;
@@ -2815,6 +2823,37 @@ void ui_display_sync(ui_display_t *disp) {
 #endif
 }
 
+void ui_set_use_clipboard_selection(int use_it) {
+  u_int count;
+
+  /* "use_clipboard = false" setting is disabled on wayland. */
+  if (use_it == 0) {
+    bl_msg_printf("use_clipboard=false is ignored on wayland.\n");
+
+    return;
+  } else if (use_clipboard == use_it) {
+    return;
+  }
+
+  if (use_clipboard == 0 && use_it == 1) {
+    /*
+     * disp->selection_owner is reset.
+     * If it isn't reset and value of 'use_clipboard' option is changed from false
+     * to true dynamically, ui_window_set_selection_owner() returns before calling
+     * wl_data_device_manager_create_data_source().
+     */
+    for (count = 0; count < num_displays; count++) {
+      if (displays[count]->selection_owner) {
+        ui_display_clear_selection(NULL, displays[count]->selection_owner);
+      }
+    }
+  }
+
+  use_clipboard = use_it;
+}
+
+int ui_is_using_clipboard_selection(void) { return use_clipboard; }
+
 /*
  * Folloing functions called from ui_window.c
  */
@@ -2848,7 +2887,7 @@ int ui_display_own_selection(ui_display_t *disp, ui_window_t *win) {
                                                wlserv->xsel_source, wlserv->serial);
   }
 
-  if (wlserv->data_device_manager) {
+  if (use_clipboard && wlserv->data_device_manager) {
     wlserv->sel_source = wl_data_device_manager_create_data_source(wlserv->data_device_manager);
     wl_data_source_offer(wlserv->sel_source, "UTF8_STRING");
     /* wl_data_source_offer(wlserv->sel_source, "COMPOUND_TEXT"); */
@@ -3152,7 +3191,7 @@ void ui_display_request_text_selection(ui_display_t *disp) {
   } else {
     ui_wlserv_t *wlserv = disp->display->wlserv;
 
-    if (wlserv->xsel_offer && wlserv->xsel_offer_mime) {
+    if (use_clipboard != 2 && wlserv->xsel_offer && wlserv->xsel_offer_mime) {
       receive_data(disp, wlserv->xsel_offer, wlserv->xsel_offer_mime, 0);
     } else if (wlserv->sel_offer && wlserv->sel_offer_mime) {
       receive_data(disp, wlserv->sel_offer, wlserv->sel_offer_mime, 1);
@@ -3280,7 +3319,14 @@ void ui_display_init_wlserv(ui_wlserv_t *wlserv) {
   wlserv->sel_fd = -1;
 }
 
-void ui_display_map(ui_display_t *disp) {
+/*
+ * dummy == is_initial_allocation()
+ * vte_terminal_realize() doesn't call ui_window_resize_with_margin()
+ * if is_initial_allocation() is true, so the size of ui_window and vt_term
+ * is incorrect and ui_window_update_all() should be never called here.
+ * (see vte_terminal_realize())
+ */
+void ui_display_map(ui_display_t *disp, int dummy) {
   if (!disp->display->buffer) {
     ui_window_t *win;
 
@@ -3316,8 +3362,9 @@ void ui_display_map(ui_display_t *disp) {
     disp->roots[0]->is_mapped = 0;
     ui_window_show(disp->roots[0], 0);
     disp->roots[0]->is_mapped = 1;
+
     if (!ui_window_resize_with_margin(disp->roots[0], disp->width, disp->height,
-                                      NOTIFY_TO_MYSELF)) {
+                                      NOTIFY_TO_MYSELF) && !dummy) {
       ui_window_update_all(disp->roots[0]);
     }
   }
