@@ -33,6 +33,7 @@ extern "C" {
 #include <pobl/bl_str.h> /* bl_str_replace */
 #include <pobl/bl_net.h>
 #include <pobl/bl_locale.h>
+#include <pobl/bl_util.h> /* BL_ARRAY_SIZE */
 
 #ifndef USE_WIN32API
 #define _XOPEN_SOURCE
@@ -593,7 +594,7 @@ static int final(vt_pty_t *pty) {
    */
   dead_network = pty_mosh->network;
 #ifdef MOSH_SIXEL
-  if (dead_network >= 0) {
+  if (dead_network->tcp_sock >= 0) {
     closesocket(dead_network->tcp_sock);
     dead_network->tcp_sock = -1; /* for dead_network->tick() not to send via tcp_sock */
   }
@@ -1043,6 +1044,10 @@ vt_pty_t *vt_pty_mosh_new(const char *cmd_path, /* If NULL, child prcess is not
   }
 #endif
 
+  if (getenv("MOSH_UDP_PORT")) {
+    argv_count += 2;
+  }
+
   u_int env_count = 0;
   if (env) {
     while (env[env_count++]);
@@ -1052,7 +1057,7 @@ vt_pty_t *vt_pty_mosh_new(const char *cmd_path, /* If NULL, child prcess is not
   if (argv_count + env_count > 0 &&
       (argv = (char**)alloca(sizeof(base_argv) + sizeof(char*) * (argv_count + env_count * 2)))) {
     memcpy(argv, base_argv, sizeof(base_argv));
-    char **argv_p = argv + sizeof(base_argv) / sizeof(base_argv[0]) - 1;
+    char **argv_p = argv + BL_ARRAY_SIZE(base_argv) - 1;
 
 #ifdef MOSH_SIXEL
     if (getenv("MOSH_NO_TCP")) {
@@ -1060,6 +1065,13 @@ vt_pty_t *vt_pty_mosh_new(const char *cmd_path, /* If NULL, child prcess is not
     }
 #endif
 
+    /* port:[port2] (Default 60000-6100) (See -p option of mosh-server) */
+    char *udp_port = getenv("MOSH_UDP_PORT");
+    if (udp_port) {
+      *(argv_p++) = "-p";
+      *(argv_p++) = udp_port;
+    }
+
     for (env_count = 0; env[env_count]; env_count++) {
       /* "COLORFGBG=default;default" breaks following environmental variables. */
       if (!strchr(env[env_count], ';')) {
