@@ -30,6 +30,7 @@ typedef struct vt_term {
    */
   char *icon_path;
   char *uri;
+  char *privkey;
   char *bidi_separators;
 
   /* vt_bidi_mode_t */ int8_t bidi_mode;
@@ -144,8 +145,8 @@ pid_t vt_term_get_pty_mode(vt_term_t *term);
 
 size_t vt_term_write(vt_term_t *term, u_char *buf, size_t len);
 
-#define vt_term_write_modified_key(term, key, modcode) \
-  ((term)->pty ? vt_parser_write_modified_key((term)->parser, key, modcode) : 0)
+#define vt_term_write_modified_key(term, key, ch, modcode)               \
+  ((term)->pty ? vt_parser_write_modified_key((term)->parser, key, ch, modcode) : 0)
 
 #define vt_term_write_special_key(term, key, modcode, is_numlock) \
   ((term)->pty ? vt_parser_write_special_key((term)->parser, key, modcode, is_numlock) : 0)
@@ -297,6 +298,8 @@ void vt_term_set_icon_path(vt_term_t *term, const char *path);
 
 #define vt_term_get_uri(term) ((term)->uri)
 
+#define vt_term_get_privkey(term) ((term)->privkey)
+
 #define vt_term_is_bracketed_paste_mode(term) \
   vt_parser_is_bracketed_paste_mode((term)->parser)
 
